
#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "fds.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "nrf_fstorage.h"
#include "nrf_fstorage_sd.h"

#include "fs_utils.h"

extern volatile uint32_t is_busy;																							/**< External global flag used for waiting for asynchronous events. */
static stored_variables_t data_struct;																				/**< Structure used for storing the data in flash. */ 

static ret_code_t fs_read_data_struct(uint32_t fs_read_address);							/**< Prototype for the fs_read_data_struct() .*/
void fs_evt_handler(nrf_fstorage_evt_t * p_evt);															/**< Prototype for fstorage event handler function. */

/**@brief Definition of the fstorage instance.
 */
NRF_FSTORAGE_DEF(nrf_fstorage_t fs_instance) =
{
    .evt_handler    = fs_evt_handler,
    .start_addr     = FS_START_ADDRESS,
    .end_addr       = FS_END_ADDRESS,
};


/**@brief Callback function for fstorage events.
 *
 * @param[in] p_evt   Pointer to the fstorage event.
 */
void fs_evt_handler(nrf_fstorage_evt_t * p_evt)
{
	switch(p_evt->id)
	{
		case NRF_FSTORAGE_EVT_WRITE_RESULT:
			is_busy--;
			break;
		
		case NRF_FSTORAGE_EVT_ERASE_RESULT:
			is_busy--;
			break;
		
		case NRF_FSTORAGE_EVT_READ_RESULT:
			break;
		
		default:
			break;
	}
}

/**@brief Function for erasing the previous mode from flash.
 *
 * @return ret_code_t  	Return of error code for determining the status of the erase operation.
 */
static ret_code_t fs_erase(uint32_t fs_erase_address)
{
	is_busy++;
	ret_code_t err_code = nrf_fstorage_erase( 
		&fs_instance,   	/* The instance to use. */
    fs_erase_address, /* The address of the flash pages to erase. */
    1, 								/* The number of pages to erase. 512 bytes */
    NULL            	/* Optional parameter, backend-dependent. */
	);
	APP_ERROR_CHECK(err_code);
	
	return(err_code);
}

/**@brief Function for reading the structure used to store the data in flash.
 *
 * @return ret_code_t  	Error code for determining the status of the read operation.
 */
static ret_code_t fs_read_data_struct(uint32_t fs_read_address)
{
	ret_code_t rc = nrf_fstorage_read(
    &fs_instance,   					/* The instance to use. */
    fs_read_address,      		/* The address in flash where to read data from. */
    &data_struct,        			/* A buffer to copy the data into. */
    sizeof(data_struct)  			/* Lenght of the data, in bytes. */
	);
	return rc;
}


/**@brief Function for writing the structure that holds the data in flash.
 *
 * @return ret_code_t   		Error code for determining the status of the write operation.
 */
static ret_code_t fs_write_data_struct(uint32_t fs_write_address)
{
	is_busy++;
	// Clear space to be written to 
	ret_code_t rc = fs_erase(fs_write_address);
	APP_ERROR_CHECK(rc);
	rc = nrf_fstorage_write(
    &fs_instance,   			/* The instance to use. */
    fs_write_address,   	/* The address in flash where to store the data. */
    &data_struct,        	/* A pointer to the data. */
    sizeof(data_struct), 	/* Length of the data, in bytes. */
    NULL            			/* Optional parameter, backend-dependent. */
	);
	APP_ERROR_CHECK(rc);		
	return rc;
}


uint32_t is_fs_busy(void)
{
	if (nrf_fstorage_is_busy(&fs_instance))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**@brief Function for reading the system modes stored in flash.
 *
 * @return uint32_t		Return of the value that is retrieved from flash.
 */
uint16_t fs_read_uuid(uint32_t fs_read_address)
{
	fs_read_data_struct(fs_read_address);
		
	return data_struct.sensor_uuid;
}


/**@brief Function for writing the system modes to flash.
 *
 * @param[in] mode   		The system mode which is to be stored in flash.
 * @return ret_code_t   Error code for determining the status of the write operation.
 */
ret_code_t fs_write_uuid(uint32_t fs_write_address, uint16_t sensor_uuid)
{
	data_struct.sensor_uuid = sensor_uuid;
	return fs_write_data_struct(fs_write_address);
}


/**@brief Initialisation function for flash storage.
 *
 * @return ret_code_t   Return of error code for determining the state of the initialisation operation.
 */
ret_code_t fs_utils_init(void)
{
	ret_code_t err_code = nrf_fstorage_init(
        &fs_instance,       /* You fstorage instance, previously defined. */
        &nrf_fstorage_sd,   /* Name of the backend. */
        NULL                /* Optional parameter, backend-dependant. */
    );
	
	err_code = fs_erase(FS_START_ADDRESS);
	APP_ERROR_CHECK(err_code);
	
	memset(&data_struct, 0, sizeof(data_struct)); //Initialise the struct used for storing data
	
	return err_code;
}

